<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
    <xsl:variable name="page_width"><xsl:value-of select="/objects/page_width"/></xsl:variable>
    <xsl:variable name="page_height"><xsl:value-of select="/objects/page_height"/></xsl:variable>
    <xsl:variable name="initial_bottom_pos"><xsl:value-of select="/objects/initial_bottom_pos"/></xsl:variable>
    <xsl:variable name="initial_left_pos"><xsl:value-of select="/objects/initial_left_pos"/></xsl:variable>
    <xsl:variable name="height_increment"><xsl:value-of select="/objects/height_incr"/></xsl:variable>
    <xsl:variable name="width_increment"><xsl:value-of select="/objects/width_incr"/></xsl:variable>
    <xsl:variable name="frame_height"><xsl:value-of select="/objects/label_height"/></xsl:variable>
    <xsl:variable name="frame_width"><xsl:value-of select="/objects/label_width"/></xsl:variable>
    <xsl:variable name="number_columns"><xsl:value-of select="/objects/cols"/></xsl:variable>
    <xsl:variable name="max_frames"><xsl:value-of select="/objects/rows * /objects/cols"/></xsl:variable>
    <xsl:variable name="printer_top"><xsl:value-of select="/objects/printer_top"/></xsl:variable>
    <xsl:variable name="printer_bottom"><xsl:value-of select="/objects/printer_bottom"/></xsl:variable>
    <xsl:variable name="printer_left"><xsl:value-of select="/objects/printer_left"/></xsl:variable>
    <xsl:variable name="printer_right"><xsl:value-of select="/objects/printer_right"/></xsl:variable>

    <xsl:template match="/">
        <xsl:apply-templates select="objects"/>
    </xsl:template>

    <xsl:template match="objects">
        <document>
            <template leftMargin="2.0cm" rightMargin="2.0cm" topMargin="2.0cm" bottomMargin="2.0cm" title="Labels" author="Generated by OpenERP">
                <xsl:attribute name="pageSize">
                    <xsl:value-of select="page_width"/><xsl:text>cm,</xsl:text><xsl:value-of select="page_height"/><xsl:text>cm</xsl:text>
                </xsl:attribute>
                <pageTemplate id="all">
                    <pageGraphics/>
                    <xsl:apply-templates select="object" mode="frames"/>
                </pageTemplate>
            </template>
            <stylesheet>
                <paraStyle name="nospace" spaceBefore="0" spaceAfter="0">
                    <xsl:attribute name="fontName">
                        <xsl:value-of select="font_type"/>
                    </xsl:attribute>
                    <xsl:attribute name="fontSize">
                        <xsl:value-of select="font_size"/>
                    </xsl:attribute>
                </paraStyle>
                <!-- In this point will be included additional report style sheets -->
                %s
            </stylesheet>
            <story>
                <xsl:apply-templates select="object" mode="story">
                    <!-- In this point will be included optional fields to sort the labels -->
                    <xsl:sort select="%s"/>
                    <xsl:sort select="%s"/>
                    <xsl:sort select="%s"/>
                </xsl:apply-templates>
            </story>
        </document>
    </xsl:template>

    <xsl:template match="object" mode="frames">
        <xsl:if test="position() &lt; $max_frames + 1">
            <xsl:variable name="width"><xsl:value-of select="$frame_width"/></xsl:variable>
            <xsl:variable name="height"><xsl:value-of select="$frame_height"/></xsl:variable>
            <xsl:variable name="x1"><xsl:value-of select="$initial_left_pos + ((position()-1) mod $number_columns) * $width_increment"/></xsl:variable>
            <xsl:variable name="y1"><xsl:value-of select="$initial_bottom_pos - floor((position()-1) div $number_columns) * $height_increment"/></xsl:variable>
            <xsl:variable name="leftPadding"><xsl:value-of select="0"/></xsl:variable>
            <xsl:variable name="bottomPadding"><xsl:value-of select="0"/></xsl:variable>
            <xsl:variable name="rightPadding"><xsl:value-of select="0"/></xsl:variable>
            <xsl:variable name="topPadding"><xsl:value-of select="0"/></xsl:variable>
            <frame>
            <xsl:choose>
                <!--Left labels over the left printer margin-->
                <xsl:when test="$x1 &lt; $printer_left">
                    <xsl:attribute name="x1">
                        <xsl:value-of select="$printer_left"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="width">
                        <xsl:value-of select="$width - ($printer_left - $x1)"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:when>
                <!--Right labels over the right printer margin-->
                <xsl:when test="$page_width - ($x1 + $width) &lt; $printer_right">
                    <xsl:attribute name="x1">
                        <xsl:value-of select="$x1"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="width">
                        <xsl:value-of select="$page_width - ($x1 + $printer_right)"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="x1">
                        <xsl:value-of select="$x1"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="width">
                        <xsl:value-of select="$width"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <!--Bottom labels over the bottom printer margin-->
                <xsl:when test="$y1 &lt; $printer_bottom">
                    <xsl:attribute name="y1">
                        <xsl:value-of select="$printer_bottom"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="height">
                        <xsl:value-of select="$height - ($printer_bottom - $y1)"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:when>
                <!--Top labels over the top printer margin-->
                <xsl:when test="$page_height - ($y1 + $height) &lt; $printer_top">
                    <xsl:attribute name="y1">
                        <xsl:value-of select="$y1"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="height">
                        <xsl:value-of select="$page_height - ($y1 + $printer_top)"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="y1">
                        <xsl:value-of select="$y1"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="height">
                        <xsl:value-of select="$height"/>
                        <xsl:text>cm</xsl:text>
                    </xsl:attribute>
                </xsl:otherwise>
            </xsl:choose>
            </frame>
        </xsl:if>
    </xsl:template>

<!-- In this point will be included the specific definition of the template -->
%s

</xsl:stylesheet>
